@@ -139,6 +139,8 @@ static int s_determine_socket_error(int error) {
 static int s_create_socket(struct aws_socket *sock, const struct aws_socket_options *options) {
 
     int fd = socket(s_convert_domain(options->domain), s_convert_type(options->type), 0);
+    int errno_value = errno; /* Always cache errno before potential side-effect */
+
     AWS_LOGF_DEBUG(
         AWS_LS_IO_SOCKET,
         "id=%p fd=%d: initializing with domain %d and type %d",
@@ -156,7 +158,7 @@ static int s_create_socket(struct aws_socket *sock, const struct aws_socket_opti
         return aws_socket_set_options(sock, options);
     }
 
-    int aws_error = s_determine_socket_error(errno);
+    int aws_error = s_determine_socket_error(errno_value);
     return aws_raise_error(aws_error);
 }
 
@@ -276,13 +278,14 @@ static int s_update_local_endpoint(struct aws_socket *socket) {
     socklen_t address_size = sizeof(address);
 
     if (getsockname(socket->io_handle.data.fd, (struct sockaddr *)&address, &address_size) != 0) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_ERROR(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: getsockname() failed with error %d",
             (void *)socket,
             socket->io_handle.data.fd,
-            errno);
-        int aws_error = s_determine_socket_error(errno);
+            errno_value);
+        int aws_error = s_determine_socket_error(errno_value);
         return aws_raise_error(aws_error);
     }
 
@@ -290,26 +293,29 @@ static int s_update_local_endpoint(struct aws_socket *socket) {
         struct sockaddr_in *s = (struct sockaddr_in *)&address;
         tmp_endpoint.port = ntohs(s->sin_port);
         if (inet_ntop(AF_INET, &s->sin_addr, tmp_endpoint.address, sizeof(tmp_endpoint.address)) == NULL) {
+            int errno_value = errno; /* Always cache errno before potential side-effect */
+
             AWS_LOGF_ERROR(
                 AWS_LS_IO_SOCKET,
                 "id=%p fd=%d: inet_ntop() failed with error %d",
                 (void *)socket,
                 socket->io_handle.data.fd,
-                errno);
-            int aws_error = s_determine_socket_error(errno);
+                errno_value);
+            int aws_error = s_determine_socket_error(errno_value);
             return aws_raise_error(aws_error);
         }
     } else if (address.ss_family == AF_INET6) {
         struct sockaddr_in6 *s = (struct sockaddr_in6 *)&address;
         tmp_endpoint.port = ntohs(s->sin6_port);
         if (inet_ntop(AF_INET6, &s->sin6_addr, tmp_endpoint.address, sizeof(tmp_endpoint.address)) == NULL) {
+            int errno_value = errno; /* Always cache errno before potential side-effect */
             AWS_LOGF_ERROR(
                 AWS_LS_IO_SOCKET,
                 "id=%p fd=%d: inet_ntop() failed with error %d",
                 (void *)socket,
                 socket->io_handle.data.fd,
-                errno);
-            int aws_error = s_determine_socket_error(errno);
+                errno_value);
+            int aws_error = s_determine_socket_error(errno_value);
             return aws_raise_error(aws_error);
         }
     } else if (address.ss_family == AF_UNIX) {
@@ -376,13 +382,14 @@ static int s_on_connection_success(struct aws_socket *socket) {
     socklen_t result_length = sizeof(connect_result);
 
     if (getsockopt(socket->io_handle.data.fd, SOL_SOCKET, SO_ERROR, &connect_result, &result_length) < 0) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_ERROR(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: failed to determine connection error %d",
             (void *)socket,
             socket->io_handle.data.fd,
-            errno);
-        int aws_error = s_determine_socket_error(errno);
+            errno_value);
+        int aws_error = s_determine_socket_error(errno_value);
         aws_raise_error(aws_error);
         s_on_connection_error(socket, aws_error);
         return AWS_OP_ERR;
@@ -549,12 +556,12 @@ static void s_run_connect_success(struct aws_task *task, void *arg, enum aws_tas
     aws_mem_release(socket_args->allocator, socket_args);
 }
 
-static inline int s_convert_pton_error(int pton_code) {
+static inline int s_convert_pton_error(int pton_code, int errno_value) {
     if (pton_code == 0) {
         return AWS_IO_SOCKET_INVALID_ADDRESS;
     }
 
-    return s_determine_socket_error(errno);
+    return s_determine_socket_error(errno_value);
 }
 
 struct socket_address {
@@ -665,6 +672,7 @@ int aws_socket_connect(
     }
 
     if (pton_err != 1) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_ERROR(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: failed to parse address %s:%d.",
@@ -672,7 +680,7 @@ int aws_socket_connect(
             socket->io_handle.data.fd,
             remote_endpoint->address,
             (int)remote_endpoint->port);
-        return aws_raise_error(s_convert_pton_error(pton_err));
+        return aws_raise_error(s_convert_pton_error(pton_err, errno_value));
     }
 
     AWS_LOGF_DEBUG(
@@ -717,8 +725,8 @@ int aws_socket_connect(
     }
 
     if (error_code) {
-        error_code = errno;
-        if (error_code == EINPROGRESS || error_code == EALREADY) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
+        if (errno_value == EINPROGRESS || errno_value == EALREADY) {
             AWS_LOGF_TRACE(
                 AWS_LS_IO_SOCKET,
                 "id=%p fd=%d: connection pending waiting on event-loop notification or timeout.",
@@ -766,8 +774,8 @@ int aws_socket_connect(
                 "id=%p fd=%d: connect failed with error code %d.",
                 (void *)socket,
                 socket->io_handle.data.fd,
-                error_code);
-            int aws_error = s_determine_socket_error(error_code);
+                errno_value);
+            int aws_error = s_determine_socket_error(errno_value);
             aws_raise_error(aws_error);
             socket->event_loop = NULL;
             socket_impl->currently_subscribed = false;
@@ -836,6 +844,7 @@ int aws_socket_bind(struct aws_socket *socket, const struct aws_socket_endpoint
     }
 
     if (pton_err != 1) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_ERROR(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: failed to parse address %s:%d.",
@@ -843,18 +852,19 @@ int aws_socket_bind(struct aws_socket *socket, const struct aws_socket_endpoint
             socket->io_handle.data.fd,
             local_endpoint->address,
             (int)local_endpoint->port);
-        return aws_raise_error(s_convert_pton_error(pton_err));
+        return aws_raise_error(s_convert_pton_error(pton_err, errno_value));
     }
 
     if (bind(socket->io_handle.data.fd, (struct sockaddr *)&address.sock_addr_types, sock_size) != 0) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_ERROR(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: bind failed with error code %d",
             (void *)socket,
             socket->io_handle.data.fd,
-            errno);
+            errno_value);
 
-        aws_raise_error(s_determine_socket_error(errno));
+        aws_raise_error(s_determine_socket_error(errno_value));
         goto error;
     }
 
@@ -916,16 +926,18 @@ int aws_socket_listen(struct aws_socket *socket, int backlog_size) {
         return AWS_OP_SUCCESS;
     }
 
+    int errno_value = errno; /* Always cache errno before potential side-effect */
+
     AWS_LOGF_ERROR(
         AWS_LS_IO_SOCKET,
         "id=%p fd=%d: listen failed with error code %d",
         (void *)socket,
         socket->io_handle.data.fd,
-        error_code);
-    error_code = errno;
+        errno_value);
+
     socket->state = ERROR;
 
-    return aws_raise_error(s_determine_socket_error(error_code));
+    return aws_raise_error(s_determine_socket_error(errno_value));
 }
 
 /* this is called by the event loop handler that was installed in start_accept(). It runs once the FD goes readable,
@@ -952,9 +964,9 @@ static void s_socket_accept_event(
 
             in_fd = accept(handle->data.fd, (struct sockaddr *)&in_addr, &in_len);
             if (in_fd == -1) {
-                int error = errno;
+                int errno_value = errno; /* Always cache errno before potential side-effect */
 
-                if (error == EAGAIN || error == EWOULDBLOCK) {
+                if (errno_value == EAGAIN || errno_value == EWOULDBLOCK) {
                     break;
                 }
 
@@ -1000,7 +1012,7 @@ static void s_socket_accept_event(
                         AWS_LS_IO_SOCKET,
                         "id=%p fd=%d:. Failed to determine remote address.",
                         (void *)socket,
-                        socket->io_handle.data.fd)
+                        socket->io_handle.data.fd);
                 }
                 new_sock->options.domain = AWS_SOCKET_IPV4;
             } else if (in_addr.ss_family == AF_INET6) {
@@ -1017,7 +1029,7 @@ static void s_socket_accept_event(
                         AWS_LS_IO_SOCKET,
                         "id=%p fd=%d:. Failed to determine remote address.",
                         (void *)socket,
-                        socket->io_handle.data.fd)
+                        socket->io_handle.data.fd);
                 }
                 new_sock->options.domain = AWS_SOCKET_IPV6;
             } else if (in_addr.ss_family == AF_UNIX) {
@@ -1224,23 +1236,25 @@ int aws_socket_set_options(struct aws_socket *socket, const struct aws_socket_op
     int option_value = 1;
     if (AWS_UNLIKELY(setsockopt(
             socket->io_handle.data.fd, SOL_SOCKET, NO_SIGNAL_SOCK_OPT, &option_value, sizeof(option_value)))) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_WARN(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: setsockopt() for NO_SIGNAL_SOCK_OPT failed with errno %d.",
             (void *)socket,
             socket->io_handle.data.fd,
-            errno);
+            errno_value);
     }
 #endif /* NO_SIGNAL_SOCK_OPT */
 
     int reuse = 1;
     if (AWS_UNLIKELY(setsockopt(socket->io_handle.data.fd, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(int)))) {
+        int errno_value = errno; /* Always cache errno before potential side-effect */
         AWS_LOGF_WARN(
             AWS_LS_IO_SOCKET,
             "id=%p fd=%d: setsockopt() for SO_REUSEADDR failed with errno %d.",
             (void *)socket,
             socket->io_handle.data.fd,
-            errno);
+            errno_value);
     }
 
     if (options->type == AWS_SOCKET_STREAM && options->domain != AWS_SOCKET_LOCAL) {
@@ -1248,36 +1262,40 @@ int aws_socket_set_options(struct aws_socket *socket, const struct aws_socket_op
             int keep_alive = 1;
             if (AWS_UNLIKELY(
                     setsockopt(socket->io_handle.data.fd, SOL_SOCKET, SO_KEEPALIVE, &keep_alive, sizeof(int)))) {
+                int errno_value = errno; /* Always cache errno before potential side-effect */
                 AWS_LOGF_WARN(
                     AWS_LS_IO_SOCKET,
                     "id=%p fd=%d: setsockopt() for enabling SO_KEEPALIVE failed with errno %d.",
                     (void *)socket,
                     socket->io_handle.data.fd,
-                    errno);
+                    errno_value);
             }
         }
 
+#if !defined(__OpenBSD__)
         if (socket->options.keep_alive_interval_sec && socket->options.keep_alive_timeout_sec) {
             int ival_in_secs = socket->options.keep_alive_interval_sec;
             if (AWS_UNLIKELY(setsockopt(
                     socket->io_handle.data.fd, IPPROTO_TCP, TCP_KEEPIDLE, &ival_in_secs, sizeof(ival_in_secs)))) {
+                int errno_value = errno; /* Always cache errno before potential side-effect */
                 AWS_LOGF_WARN(
                     AWS_LS_IO_SOCKET,
                     "id=%p fd=%d: setsockopt() for enabling TCP_KEEPIDLE for TCP failed with errno %d.",
                     (void *)socket,
                     socket->io_handle.data.fd,
-                    errno);
+                    errno_value);
             }
 
             ival_in_secs = socket->options.keep_alive_timeout_sec;
             if (AWS_UNLIKELY(setsockopt(
                     socket->io_handle.data.fd, IPPROTO_TCP, TCP_KEEPINTVL, &ival_in_secs, sizeof(ival_in_secs)))) {
+                int errno_value = errno; /* Always cache errno before potential side-effect */
                 AWS_LOGF_WARN(
                     AWS_LS_IO_SOCKET,
                     "id=%p fd=%d: setsockopt() for enabling TCP_KEEPINTVL for TCP failed with errno %d.",
                     (void *)socket,
                     socket->io_handle.data.fd,
-                    errno);
+                    errno_value);
             }
         }
 
@@ -1285,14 +1303,16 @@ int aws_socket_set_options(struct aws_socket *socket, const struct aws_socket_op
             int max_probes = socket->options.keep_alive_max_failed_probes;
             if (AWS_UNLIKELY(
                     setsockopt(socket->io_handle.data.fd, IPPROTO_TCP, TCP_KEEPCNT, &max_probes, sizeof(max_probes)))) {
+                int errno_value = errno; /* Always cache errno before potential side-effect */
                 AWS_LOGF_WARN(
                     AWS_LS_IO_SOCKET,
                     "id=%p fd=%d: setsockopt() for enabling TCP_KEEPCNT for TCP failed with errno %d.",
                     (void *)socket,
                     socket->io_handle.data.fd,
-                    errno);
+                    errno_value);
             }
         }
+#endif /* __OpenBSD__ */
     }
 
     return AWS_OP_SUCCESS;
@@ -1444,7 +1464,8 @@ int aws_socket_shutdown_dir(struct aws_socket *socket, enum aws_channel_directio
     AWS_LOGF_DEBUG(
         AWS_LS_IO_SOCKET, "id=%p fd=%d: shutting down in direction %d", (void *)socket, socket->io_handle.data.fd, dir);
     if (shutdown(socket->io_handle.data.fd, how)) {
-        int aws_error = s_determine_socket_error(errno);
+        int errno_value = errno; /* Always cache errno before potential side-effect */
+        int aws_error = s_determine_socket_error(errno_value);
         return aws_raise_error(aws_error);
     }
 
@@ -1539,6 +1560,7 @@ static int s_process_socket_write_requests(struct aws_socket *socket, struct soc
 
         ssize_t written = send(
             socket->io_handle.data.fd, write_request->cursor_cpy.ptr, write_request->cursor_cpy.len, NO_SIGNAL_SEND);
+        int errno_value = errno; /* Always cache errno before potential side-effect */
 
         AWS_LOGF_TRACE(
             AWS_LS_IO_SOCKET,
@@ -1548,14 +1570,13 @@ static int s_process_socket_write_requests(struct aws_socket *socket, struct soc
             (int)written);
 
         if (written < 0) {
-            int error = errno;
-            if (error == EAGAIN) {
+            if (errno_value == EAGAIN) {
                 AWS_LOGF_TRACE(
                     AWS_LS_IO_SOCKET, "id=%p fd=%d: returned would block", (void *)socket, socket->io_handle.data.fd);
                 break;
             }
 
-            if (error == EPIPE) {
+            if (errno_value == EPIPE) {
                 AWS_LOGF_DEBUG(
                     AWS_LS_IO_SOCKET,
                     "id=%p fd=%d: already closed before write",
@@ -1573,8 +1594,8 @@ static int s_process_socket_write_requests(struct aws_socket *socket, struct soc
                 "id=%p fd=%d: write error with error code %d",
                 (void *)socket,
                 socket->io_handle.data.fd,
-                error);
-            aws_error = s_determine_socket_error(error);
+                errno_value);
+            aws_error = s_determine_socket_error(errno_value);
             aws_raise_error(aws_error);
             break;
         }
@@ -1780,7 +1801,7 @@ int aws_socket_read(struct aws_socket *socket, struct aws_byte_buf *buffer, size
     }
 
     ssize_t read_val = read(socket->io_handle.data.fd, buffer->buffer + buffer->len, buffer->capacity - buffer->len);
-    int error = errno;
+    int errno_value = errno; /* Always cache errno before potential side-effect */
 
     AWS_LOGF_TRACE(
         AWS_LS_IO_SOCKET, "id=%p fd=%d: read of %d", (void *)socket, socket->io_handle.data.fd, (int)read_val);
@@ -1805,20 +1826,20 @@ int aws_socket_read(struct aws_socket *socket, struct aws_byte_buf *buffer, size
     }
 
 #if defined(EWOULDBLOCK)
-    if (error == EAGAIN || error == EWOULDBLOCK) {
+    if (errno_value == EAGAIN || errno_value == EWOULDBLOCK) {
 #else
-    if (error == EAGAIN) {
+    if (errno_value == EAGAIN) {
 #endif
         AWS_LOGF_TRACE(AWS_LS_IO_SOCKET, "id=%p fd=%d: read would block", (void *)socket, socket->io_handle.data.fd);
         return aws_raise_error(AWS_IO_READ_WOULD_BLOCK);
     }
 
-    if (error == EPIPE || error == ECONNRESET) {
+    if (errno_value == EPIPE || errno_value == ECONNRESET) {
         AWS_LOGF_INFO(AWS_LS_IO_SOCKET, "id=%p fd=%d: socket is closed.", (void *)socket, socket->io_handle.data.fd);
         return aws_raise_error(AWS_IO_SOCKET_CLOSED);
     }
 
-    if (error == ETIMEDOUT) {
+    if (errno_value == ETIMEDOUT) {
         AWS_LOGF_ERROR(AWS_LS_IO_SOCKET, "id=%p fd=%d: socket timed out.", (void *)socket, socket->io_handle.data.fd);
         return aws_raise_error(AWS_IO_SOCKET_TIMEOUT);
     }
@@ -1828,8 +1849,8 @@ int aws_socket_read(struct aws_socket *socket, struct aws_byte_buf *buffer, size
         "id=%p fd=%d: read failed with error: %s",
         (void *)socket,
         socket->io_handle.data.fd,
-        strerror(error));
-    return aws_raise_error(s_determine_socket_error(error));
+        strerror(errno_value));
+    return aws_raise_error(s_determine_socket_error(errno_value));
 }
 
 int aws_socket_write(
