@@ -192,6 +192,7 @@ struct aws_mqtt_client_connection {
     struct aws_socket_options socket_options;
     struct aws_http_proxy_config *http_proxy_config;
     struct aws_event_loop *loop;
+    struct aws_host_resolution_config host_resolution_config;
 
     /* Connect parameters */
     struct aws_byte_buf client_id;
@@ -208,10 +209,15 @@ struct aws_mqtt_client_connection {
         struct aws_byte_buf payload;
     } will;
     struct {
-        uint64_t current_sec;                 /* seconds */
-        uint64_t min_sec;                     /* seconds */
-        uint64_t max_sec;                     /* seconds */
-        uint64_t next_attempt_reset_timer_ns; /* nanoseconds */
+        uint64_t current_sec; /* seconds */
+        uint64_t min_sec;     /* seconds */
+        uint64_t max_sec;     /* seconds */
+
+        /*
+         * Invariant: this is always zero except when the current MQTT channel has received a successful connack
+         * and is not yet shutdown.  During that interval, it is the timestamp the connack was received.
+         */
+        uint64_t channel_successful_connack_timestamp_ns;
     } reconnect_timeouts;
 
     /* User connection callbacks */
