@@ -110,20 +110,14 @@ static int s_packet_handler_connack(
     uint64_t now = 0;
     aws_high_res_clock_get_ticks(&now);
 
-    /*
-     * Only reset the duration of the reconnect timer to min if this connect is happening past
-     * the previously set next_attempt_reset_timer value. The next reset value will be 10 seconds after the next
-     * connection attempt
-     */
-    if (connection->reconnect_timeouts.next_attempt_reset_timer_ns < now) {
-        connection->reconnect_timeouts.current_sec = connection->reconnect_timeouts.min_sec;
-    }
-    connection->reconnect_timeouts.next_attempt_reset_timer_ns =
-        now + 10000000000 +
-        aws_timestamp_convert(
-            connection->reconnect_timeouts.current_sec, AWS_TIMESTAMP_SECS, AWS_TIMESTAMP_NANOS, NULL);
-
     if (connack.connect_return_code == AWS_MQTT_CONNECT_ACCEPTED) {
+
+        /*
+         * This was a successful MQTT connection establishment, record the time so that channel shutdown
+         * can make a good decision about reconnect backoff reset.
+         */
+        connection->reconnect_timeouts.channel_successful_connack_timestamp_ns = now;
+
         /* If successfully connected, schedule all pending tasks */
         AWS_LOGF_TRACE(
             AWS_LS_MQTT_CLIENT, "id=%p: connection was accepted processing offline requests.", (void *)connection);
@@ -721,10 +715,17 @@ static void s_request_outgoing_task(struct aws_channel_task *task, void *arg, en
                 "%" PRIu16 ". will be retried",
                 (void *)task,
                 request->packet_id);
+
             /* put it into the offline queue. */
             { /* BEGIN CRITICAL SECTION */
                 mqtt_connection_lock_synced_data(connection);
+
+                /* Set the status as incomplete */
+                aws_mqtt_connection_statistics_change_operation_statistic_state(
+                    connection, request, AWS_MQTT_OSS_INCOMPLETE);
+
                 aws_linked_list_push_back(&connection->synced_data.pending_requests_list, &request->list_node);
+
                 mqtt_connection_unlock_synced_data(connection);
             } /* END CRITICAL SECTION */
         } else {
@@ -734,13 +735,19 @@ static void s_request_outgoing_task(struct aws_channel_task *task, void *arg, en
                 "%" PRIu16 ". will NOT be retried, will be cancelled",
                 (void *)task,
                 request->packet_id);
+
             /* Fire the callback and clean up the memory, as the connection get destroyed. */
             if (request->on_complete) {
                 request->on_complete(
                     connection, request->packet_id, AWS_ERROR_MQTT_NOT_CONNECTED, request->on_complete_ud);
             }
+
             { /* BEGIN CRITICAL SECTION */
                 mqtt_connection_lock_synced_data(connection);
+
+                /* Cancel the request in the operation statistics */
+                aws_mqtt_connection_statistics_change_operation_statistic_state(connection, request, AWS_MQTT_OSS_NONE);
+
                 aws_hash_table_remove(
                     &connection->synced_data.outstanding_requests_table, &request->packet_id, NULL, NULL);
                 aws_memory_pool_release(&connection->synced_data.requests_pool, request);
@@ -772,13 +779,20 @@ static void s_request_outgoing_task(struct aws_channel_task *task, void *arg, en
                 "id=%p: sending request %" PRIu16 " complete, invoking on_complete callback.",
                 (void *)request->connection,
                 request->packet_id);
+
             /* If the send_request function reports the request is complete,
              * remove from the hash table and call the callback. */
             if (request->on_complete) {
                 request->on_complete(connection, request->packet_id, error_code, request->on_complete_ud);
             }
+
             { /* BEGIN CRITICAL SECTION */
                 mqtt_connection_lock_synced_data(connection);
+
+                /* Set the request as complete in the operation statistics */
+                aws_mqtt_connection_statistics_change_operation_statistic_state(
+                    request->connection, request, AWS_MQTT_OSS_NONE);
+
                 aws_hash_table_remove(
                     &connection->synced_data.outstanding_requests_table, &request->packet_id, NULL, NULL);
                 aws_memory_pool_release(&connection->synced_data.requests_pool, request);
@@ -792,6 +806,17 @@ static void s_request_outgoing_task(struct aws_channel_task *task, void *arg, en
                 "id=%p: request %" PRIu16 " sent, but waiting on an acknowledgement from peer.",
                 (void *)request->connection,
                 request->packet_id);
+
+            { /* BEGIN CRITICAL SECTION */
+                mqtt_connection_lock_synced_data(connection);
+
+                /* Set the request as incomplete and un-acked in the operation statistics */
+                aws_mqtt_connection_statistics_change_operation_statistic_state(
+                    request->connection, request, AWS_MQTT_OSS_INCOMPLETE | AWS_MQTT_OSS_UNACKED);
+
+                mqtt_connection_unlock_synced_data(connection);
+            } /* END CRITICAL SECTION */
+
             /* Put the request into the ongoing list */
             aws_linked_list_push_back(&connection->thread_data.ongoing_requests_list, &request->list_node);
             break;
@@ -804,7 +829,8 @@ uint16_t mqtt_create_request(
     void *send_request_ud,
     aws_mqtt_op_complete_fn *on_complete,
     void *on_complete_ud,
-    bool noRetry) {
+    bool noRetry,
+    uint64_t packet_size) {
 
     AWS_ASSERT(connection);
     AWS_ASSERT(send_request);
@@ -899,6 +925,7 @@ uint16_t mqtt_create_request(
         next_request->send_request_ud = send_request_ud;
         next_request->on_complete = on_complete;
         next_request->on_complete_ud = on_complete_ud;
+        next_request->packet_size = packet_size;
         aws_channel_task_init(
             &next_request->outgoing_task, s_request_outgoing_task, next_request, "mqtt_outgoing_request_task");
         if (connection->synced_data.state != AWS_MQTT_CLIENT_STATE_CONNECTED) {
@@ -911,8 +938,16 @@ uint16_t mqtt_create_request(
             /* keep the channel alive until the task is scheduled */
             aws_channel_acquire_hold(channel);
         }
+
+        if (next_request && next_request->packet_size > 0) {
+            /* Set the status as incomplete */
+            aws_mqtt_connection_statistics_change_operation_statistic_state(
+                next_request->connection, next_request, AWS_MQTT_OSS_INCOMPLETE);
+        }
+
         mqtt_connection_unlock_synced_data(connection);
     } /* END CRITICAL SECTION */
+
     if (should_schedule_task) {
         AWS_LOGF_TRACE(
             AWS_LS_MQTT_CLIENT,
@@ -951,6 +986,10 @@ void mqtt_request_complete(struct aws_mqtt_client_connection *connection, int er
             on_complete = request->on_complete;
             on_complete_ud = request->on_complete_ud;
 
+            /* Set the status as complete */
+            aws_mqtt_connection_statistics_change_operation_statistic_state(
+                request->connection, request, AWS_MQTT_OSS_NONE);
+
             /* clean up request resources */
             aws_hash_table_remove_element(&connection->synced_data.outstanding_requests_table, elem);
             /* remove the request from the list, which is thread_data.ongoing_requests_list */
