@@ -7,6 +7,7 @@
 
 #include <aws/common/byte_buf.h>
 #include <aws/common/device_random.h>
+#include <aws/common/encoding.h>
 #include <inttypes.h>
 
 uint8_t aws_mqtt5_compute_fixed_header_byte1(enum aws_mqtt5_packet_type packet_type, uint8_t flags) {
@@ -270,6 +271,8 @@ const char *aws_mqtt5_client_session_behavior_type_to_c_string(
             return "Clean session always";
         case AWS_MQTT5_CSBT_REJOIN_POST_SUCCESS:
             return "Attempt to resume a session after initial connection success";
+        case AWS_MQTT5_CSBT_REJOIN_ALWAYS:
+            return "Always attempt to resume a session";
         default:
             return "Unknown session behavior";
     }
@@ -291,6 +294,9 @@ const char *aws_mqtt5_outbound_topic_alias_behavior_type_to_c_string(
             return "User-controlled outbound topic aliasing behavior";
         case AWS_MQTT5_COTABT_LRU:
             return "LRU caching outbound topic aliasing behavior";
+        case AWS_MQTT5_COTABT_DISABLED:
+            return "Outbound topic aliasing disabled";
+
         default:
             return "Unknown outbound topic aliasing behavior";
     }
@@ -299,7 +305,7 @@ const char *aws_mqtt5_outbound_topic_alias_behavior_type_to_c_string(
 enum aws_mqtt5_client_outbound_topic_alias_behavior_type aws_mqtt5_outbound_topic_alias_behavior_type_to_non_default(
     enum aws_mqtt5_client_outbound_topic_alias_behavior_type outbound_aliasing_behavior) {
     if (outbound_aliasing_behavior == AWS_MQTT5_COTABT_DEFAULT) {
-        return AWS_MQTT5_COTABT_LRU;
+        return AWS_MQTT5_COTABT_DISABLED;
     }
 
     return outbound_aliasing_behavior;
@@ -532,3 +538,37 @@ bool aws_mqtt_is_topic_filter_shared_subscription(struct aws_byte_cursor topic_c
 
     return true;
 }
+
+/* UTF-8 encoded string validation respect to [MQTT-1.5.3-2]. */
+static int aws_mqtt5_utf8_decoder(uint32_t codepoint, void *user_data) {
+    (void)user_data;
+    /* U+0000 - A UTF-8 Encoded String MUST NOT include an encoding of the null character U+0000. [MQTT-1.5.4-2]
+     * U+0001..U+001F control characters are not valid
+     */
+    if (AWS_UNLIKELY(codepoint <= 0x001F)) {
+        return aws_raise_error(AWS_ERROR_MQTT5_INVALID_UTF8_STRING);
+    }
+
+    /* U+007F..U+009F control characters are not valid */
+    if (AWS_UNLIKELY((codepoint >= 0x007F) && (codepoint <= 0x009F))) {
+        return aws_raise_error(AWS_ERROR_MQTT5_INVALID_UTF8_STRING);
+    }
+
+    /* Unicode non-characters are not valid: https://www.unicode.org/faq/private_use.html#nonchar1 */
+    if (AWS_UNLIKELY((codepoint & 0x00FFFF) >= 0x00FFFE)) {
+        return aws_raise_error(AWS_ERROR_MQTT5_INVALID_UTF8_STRING);
+    }
+    if (AWS_UNLIKELY(codepoint >= 0xFDD0 && codepoint <= 0xFDEF)) {
+        return aws_raise_error(AWS_ERROR_MQTT5_INVALID_UTF8_STRING);
+    }
+
+    return AWS_ERROR_SUCCESS;
+}
+
+struct aws_utf8_decoder_options g_aws_mqtt5_utf8_decoder_options = {
+    .on_codepoint = aws_mqtt5_utf8_decoder,
+};
+
+int aws_mqtt5_validate_utf8_text(struct aws_byte_cursor text) {
+    return aws_decode_utf8(text, &g_aws_mqtt5_utf8_decoder_options);
+}
