@@ -8,7 +8,7 @@
 #include "aws/s3/private/s3_list_parts.h"
 #include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
-#include <aws/common/json.h>
+#include <aws/common/encoding.h>
 #include <aws/common/string.h>
 #include <aws/io/stream.h>
 
@@ -38,7 +38,14 @@ static void s_s3_auto_ranged_put_request_finished(
     struct aws_s3_request *request,
     int error_code);
 
-static int s_s3_auto_ranged_put_pause(struct aws_s3_meta_request *meta_request, struct aws_string **resume_token);
+static void s_s3_auto_ranged_put_send_request_finish(
+    struct aws_s3_connection *connection,
+    struct aws_http_stream *stream,
+    int error_code);
+
+static int s_s3_auto_ranged_put_pause(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_meta_request_resume_token **resume_token);
 
 static bool s_process_part_info(const struct aws_s3_part_info *info, void *user_data) {
     struct aws_s3_auto_ranged_put *auto_ranged_put = user_data;
@@ -68,7 +75,9 @@ static bool s_process_part_info(const struct aws_s3_part_info *info, void *user_
 
     if (checksum_cur) {
         aws_byte_buf_init_copy_from_cursor(
-            &auto_ranged_put->checksums_list[info->part_number - 1], auto_ranged_put->base.allocator, *checksum_cur);
+            &auto_ranged_put->encoded_checksum_list[info->part_number - 1],
+            auto_ranged_put->base.allocator,
+            *checksum_cur);
     }
 
     aws_array_list_set_at(&auto_ranged_put->synced_data.etag_list, &etag, info->part_number - 1);
@@ -76,107 +85,87 @@ static bool s_process_part_info(const struct aws_s3_part_info *info, void *user_
     return true;
 }
 
-static int s_parse_resume_info_from_token(
-    struct aws_allocator *allocator,
-    const struct aws_byte_cursor *resume_token,
-    struct aws_string **upload_id_out,
-    size_t *part_size_out,
-    uint32_t *total_num_parts_out) {
+/*
+ * Validates token and updates part variables. Noop if token is null.
+ */
+static int s_try_update_part_info_from_resume_token(
+    uint64_t content_length,
+    const struct aws_s3_meta_request_resume_token *resume_token,
+    size_t *out_part_size,
+    uint32_t *out_total_num_parts) {
 
     if (!resume_token) {
         return AWS_OP_SUCCESS;
     }
 
-    struct aws_json_value *root = aws_json_value_new_from_string(allocator, *resume_token);
-
-    struct aws_json_value *type_node = aws_json_value_get_from_object(root, aws_byte_cursor_from_c_str("type"));
-
-    struct aws_json_value *multipart_upload_id_node =
-        aws_json_value_get_from_object(root, aws_byte_cursor_from_c_str("multipart_upload_id"));
-
-    struct aws_json_value *partition_size_node =
-        aws_json_value_get_from_object(root, aws_byte_cursor_from_c_str("partition_size"));
-
-    struct aws_json_value *total_num_parts_node =
-        aws_json_value_get_from_object(root, aws_byte_cursor_from_c_str("total_num_parts"));
-
-    double partition_size_value = 0.0;
-    double total_num_parts_value = 0.0;
-    struct aws_byte_cursor multipart_upload_id_value;
-    struct aws_byte_cursor type_value;
-
-    if (!multipart_upload_id_node || !partition_size_node || !total_num_parts_node || !type_node ||
-        aws_json_value_get_number(partition_size_node, &partition_size_value) ||
-        aws_json_value_get_number(total_num_parts_node, &total_num_parts_value) ||
-        aws_json_value_get_string(multipart_upload_id_node, &multipart_upload_id_value) ||
-        aws_json_value_get_string(type_node, &type_value)) {
-        AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Could not load persisted state. Invalid token.");
-        goto invalid_argument_cleanup;
-    }
-
-    if (!aws_byte_cursor_eq_c_str(&type_value, "AWS_S3_META_REQUEST_TYPE_PUT_OBJECT")) {
+    if (resume_token->type != AWS_S3_META_REQUEST_TYPE_PUT_OBJECT) {
         AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Could not load persisted state. Invalid token type.");
         goto invalid_argument_cleanup;
     }
 
-    if ((size_t)partition_size_value < g_s3_min_upload_part_size) {
+    if (resume_token->multipart_upload_id == NULL) {
+        AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Could not load persisted state. Multipart upload id missing.");
+        goto invalid_argument_cleanup;
+    }
+
+    if (resume_token->part_size < g_s3_min_upload_part_size) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST,
             "Could not create resume auto-ranged-put meta request; part size of %" PRIu64
             " specified in the token is below minimum threshold for multi-part.",
-            (uint64_t)partition_size_value);
+            (uint64_t)resume_token->part_size);
 
         goto invalid_argument_cleanup;
     }
 
-    if ((uint32_t)total_num_parts_value > g_s3_max_num_upload_parts) {
+    if ((uint32_t)resume_token->total_num_parts > g_s3_max_num_upload_parts) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST,
             "Could not create resume auto-ranged-put meta request; total number of parts %" PRIu32
             " specified in the token is too large for platform.",
-            (uint32_t)total_num_parts_value);
+            (uint32_t)resume_token->total_num_parts);
 
         goto invalid_argument_cleanup;
     }
 
-    *upload_id_out = aws_string_new_from_cursor(allocator, &multipart_upload_id_value);
-    *part_size_out = (size_t)partition_size_value;
-    *total_num_parts_out = (uint32_t)total_num_parts_value;
+    uint32_t num_parts = (uint32_t)(content_length / resume_token->part_size);
 
-    aws_json_value_destroy(root);
+    if ((content_length % resume_token->part_size) > 0) {
+        ++num_parts;
+    }
+
+    if (resume_token->total_num_parts != num_parts) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "Could not create auto-ranged-put meta request; persisted number of parts %zu"
+            " does not match expected number of parts based on length of the body.",
+            resume_token->total_num_parts);
+
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    *out_part_size = resume_token->part_size;
+    *out_total_num_parts = (uint32_t)resume_token->total_num_parts;
 
     return AWS_OP_SUCCESS;
 
 invalid_argument_cleanup:
-    aws_json_value_destroy(root);
     return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
 }
 
 /**
- * Loads the persistable state used to resume an upload that was previously paused.
+ * Initializes state necessary to resume upload. Noop if token is null.
  */
-static int s_load_persistable_state(
+static int s_try_init_resume_state_from_persisted_data(
     struct aws_allocator *allocator,
     struct aws_s3_auto_ranged_put *auto_ranged_put,
-    uint64_t content_length,
-    struct aws_string *upload_id,
-    size_t part_size,
-    uint32_t total_num_parts) {
+    const struct aws_s3_meta_request_resume_token *resume_token) {
 
-    uint32_t num_parts = (uint32_t)(content_length / part_size);
-
-    if ((content_length % part_size) > 0) {
-        ++num_parts;
-    }
-
-    if (total_num_parts != num_parts) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_META_REQUEST,
-            "Could not create auto-ranged-put meta request; persisted number of parts %" PRIu32
-            " does not match expected number of parts based on length of the body.",
-            total_num_parts);
-
-        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    if (resume_token == NULL) {
+        auto_ranged_put->synced_data.list_parts_operation = NULL;
+        auto_ranged_put->synced_data.list_parts_state.completed = true;
+        auto_ranged_put->synced_data.list_parts_state.started = true;
+        return AWS_OP_SUCCESS;
     }
 
     struct aws_byte_cursor request_path;
@@ -189,7 +178,7 @@ static int s_load_persistable_state(
     auto_ranged_put->synced_data.num_parts_completed = 0;
     auto_ranged_put->synced_data.create_multipart_upload_sent = true;
     auto_ranged_put->synced_data.create_multipart_upload_completed = true;
-    auto_ranged_put->upload_id = upload_id;
+    auto_ranged_put->upload_id = aws_string_clone_or_reuse(allocator, resume_token->multipart_upload_id);
 
     struct aws_s3_list_parts_params list_parts_params = {
         .key = request_path,
@@ -201,7 +190,7 @@ static int s_load_persistable_state(
     auto_ranged_put->synced_data.list_parts_operation = aws_s3_list_parts_operation_new(allocator, &list_parts_params);
 
     struct aws_http_headers *needed_response_headers = aws_http_headers_new(allocator);
-    const size_t copy_header_count = sizeof(s_create_multipart_upload_copy_headers) / sizeof(struct aws_byte_cursor);
+    const size_t copy_header_count = AWS_ARRAY_SIZE(s_create_multipart_upload_copy_headers);
     struct aws_http_headers *initial_headers =
         aws_http_message_get_headers(auto_ranged_put->base.initial_request_message);
 
@@ -212,7 +201,7 @@ static int s_load_persistable_state(
         struct aws_byte_cursor header_value;
         AWS_ZERO_STRUCT(header_value);
 
-        if (!aws_http_headers_get(initial_headers, *header_name, &header_value)) {
+        if (aws_http_headers_get(initial_headers, *header_name, &header_value) == AWS_OP_SUCCESS) {
             aws_http_headers_set(needed_response_headers, *header_name, header_value);
         }
     }
@@ -224,7 +213,7 @@ static int s_load_persistable_state(
 
 static struct aws_s3_meta_request_vtable s_s3_auto_ranged_put_vtable = {
     .update = s_s3_auto_ranged_put_update,
-    .send_request_finish = aws_s3_meta_request_send_request_finish_default,
+    .send_request_finish = s_s3_auto_ranged_put_send_request_finish,
     .prepare_request = s_s3_auto_ranged_put_prepare_request,
     .init_signing_date_time = aws_s3_meta_request_init_signing_date_time_default,
     .sign_request = aws_s3_meta_request_sign_request_default,
@@ -250,15 +239,13 @@ struct aws_s3_meta_request *aws_s3_meta_request_auto_ranged_put_new(
     AWS_PRECONDITION(options->message);
     AWS_PRECONDITION(aws_http_message_get_body_stream(options->message));
 
+    if (s_try_update_part_info_from_resume_token(content_length, options->resume_token, &part_size, &num_parts)) {
+        return NULL;
+    }
+
     struct aws_s3_auto_ranged_put *auto_ranged_put =
         aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_auto_ranged_put));
 
-    struct aws_string *multipart_upload_id = NULL;
-    if (s_parse_resume_info_from_token(
-            allocator, options->resume_token, &multipart_upload_id, &part_size, &num_parts)) {
-        goto error_clean_up;
-    }
-
     if (aws_s3_meta_request_init_base(
             allocator,
             client,
@@ -269,31 +256,28 @@ struct aws_s3_meta_request *aws_s3_meta_request_auto_ranged_put_new(
             auto_ranged_put,
             &s_s3_auto_ranged_put_vtable,
             &auto_ranged_put->base)) {
-        goto error_clean_up;
+        aws_mem_release(allocator, auto_ranged_put);
+        return NULL;
     }
 
     auto_ranged_put->content_length = content_length;
     auto_ranged_put->synced_data.total_num_parts = num_parts;
     auto_ranged_put->upload_id = NULL;
+    auto_ranged_put->resume_token = options->resume_token;
+
+    aws_s3_meta_request_resume_token_acquire(auto_ranged_put->resume_token);
 
     auto_ranged_put->threaded_update_data.next_part_number = 1;
     auto_ranged_put->prepare_data.num_parts_read_from_stream = 0;
 
-    if (options->resume_token) {
-        if (s_load_persistable_state(
-                allocator, auto_ranged_put, content_length, multipart_upload_id, part_size, num_parts)) {
-            goto error_clean_up;
-        }
-    } else {
-        auto_ranged_put->synced_data.list_parts_operation = NULL;
-        auto_ranged_put->synced_data.list_parts_sent = true;
-        auto_ranged_put->synced_data.list_parts_completed = true;
-    }
-
     struct aws_string **etag_c_array = aws_mem_calloc(allocator, sizeof(struct aws_string *), num_parts);
     aws_array_list_init_static(
         &auto_ranged_put->synced_data.etag_list, etag_c_array, num_parts, sizeof(struct aws_string *));
-    auto_ranged_put->checksums_list = aws_mem_calloc(allocator, sizeof(struct aws_byte_buf), num_parts);
+    auto_ranged_put->encoded_checksum_list = aws_mem_calloc(allocator, sizeof(struct aws_byte_buf), num_parts);
+
+    if (s_try_init_resume_state_from_persisted_data(allocator, auto_ranged_put, options->resume_token)) {
+        goto error_clean_up;
+    }
 
     AWS_LOGF_DEBUG(
         AWS_LS_S3_META_REQUEST, "id=%p Created new Auto-Ranged Put Meta Request.", (void *)&auto_ranged_put->base);
@@ -301,9 +285,7 @@ struct aws_s3_meta_request *aws_s3_meta_request_auto_ranged_put_new(
     return &auto_ranged_put->base;
 
 error_clean_up:
-    aws_string_destroy(multipart_upload_id);
-    aws_mem_release(allocator, auto_ranged_put);
-
+    aws_s3_meta_request_release(&auto_ranged_put->base);
     return NULL;
 }
 
@@ -317,23 +299,24 @@ static void s_s3_meta_request_auto_ranged_put_destroy(struct aws_s3_meta_request
     aws_string_destroy(auto_ranged_put->upload_id);
     auto_ranged_put->upload_id = NULL;
 
+    auto_ranged_put->resume_token = aws_s3_meta_request_resume_token_release(auto_ranged_put->resume_token);
+
     aws_s3_paginated_operation_release(auto_ranged_put->synced_data.list_parts_operation);
 
-    for (size_t etag_index = 0; etag_index < aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-         ++etag_index) {
+    for (size_t etag_index = 0; etag_index < auto_ranged_put->synced_data.total_num_parts; ++etag_index) {
         struct aws_string *etag = NULL;
 
         aws_array_list_get_at(&auto_ranged_put->synced_data.etag_list, &etag, etag_index);
         aws_string_destroy(etag);
     }
+
     aws_string_destroy(auto_ranged_put->synced_data.list_parts_continuation_token);
-    /* probably just better to store num parts in the meta-request */
-    for (size_t checksum_index = 0; checksum_index < aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-         ++checksum_index) {
-        aws_byte_buf_clean_up(&auto_ranged_put->checksums_list[checksum_index]);
+
+    for (size_t checksum_index = 0; checksum_index < auto_ranged_put->synced_data.total_num_parts; ++checksum_index) {
+        aws_byte_buf_clean_up(&auto_ranged_put->encoded_checksum_list[checksum_index]);
     }
     aws_mem_release(meta_request->allocator, auto_ranged_put->synced_data.etag_list.data);
-    aws_mem_release(meta_request->allocator, auto_ranged_put->checksums_list);
+    aws_mem_release(meta_request->allocator, auto_ranged_put->encoded_checksum_list);
     aws_array_list_clean_up(&auto_ranged_put->synced_data.etag_list);
     aws_http_headers_release(auto_ranged_put->synced_data.needed_response_headers);
     aws_mem_release(meta_request->allocator, auto_ranged_put);
@@ -357,20 +340,32 @@ static bool s_s3_auto_ranged_put_update(
 
         if (!aws_s3_meta_request_has_finish_result_synced(meta_request)) {
             /* If resuming and list part has not be sent, do it now. */
-            if (!auto_ranged_put->synced_data.list_parts_sent) {
+            if (!auto_ranged_put->synced_data.list_parts_state.started) {
                 request = aws_s3_request_new(
                     meta_request,
                     AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_LIST_PARTS,
                     0,
                     AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS);
 
-                auto_ranged_put->synced_data.list_parts_sent = true;
+                auto_ranged_put->synced_data.list_parts_state.started = true;
 
                 goto has_work_remaining;
             }
 
-            /* waiting on list parts to finish. */
-            if (!auto_ranged_put->synced_data.list_parts_completed) {
+            if (auto_ranged_put->synced_data.list_parts_state.continues) {
+                /* If list parts need to continue, send another list parts request. */
+                AWS_ASSERT(auto_ranged_put->synced_data.list_parts_continuation_token != NULL);
+                request = aws_s3_request_new(
+                    meta_request,
+                    AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_LIST_PARTS,
+                    0,
+                    AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS);
+                auto_ranged_put->synced_data.list_parts_state.continues = false;
+                goto has_work_remaining;
+            }
+
+            if (!auto_ranged_put->synced_data.list_parts_state.completed) {
+                /* waiting on list parts to finish. */
                 goto has_work_remaining;
             }
 
@@ -416,7 +411,7 @@ static bool s_s3_auto_ranged_put_update(
                 // Something went really wrong. we still have parts to send, but have etags for all parts
                 AWS_FATAL_ASSERT(
                     auto_ranged_put->threaded_update_data.next_part_number <=
-                    auto_ranged_put->synced_data.total_num_parts)
+                    auto_ranged_put->synced_data.total_num_parts);
 
                 if ((flags & AWS_S3_META_REQUEST_UPDATE_FLAG_CONSERVATIVE) != 0) {
                     uint32_t num_parts_in_flight =
@@ -519,6 +514,11 @@ static bool s_s3_auto_ranged_put_update(
                 if (auto_ranged_put->upload_id == NULL) {
                     goto no_work_remaining;
                 }
+                if (auto_ranged_put->base.synced_data.finish_result.error_code == AWS_ERROR_SUCCESS) {
+                    /* Not sending abort when success even if we haven't sent complete MPU, in case we resume after MPU
+                     * already completed. */
+                    goto no_work_remaining;
+                }
 
                 request = aws_s3_request_new(
                     meta_request,
@@ -586,6 +586,69 @@ static size_t s_compute_request_body_size(struct aws_s3_meta_request *meta_reque
     return request_body_size;
 }
 
+static int s_verify_part_matches_checksum(
+    struct aws_allocator *allocator,
+    struct aws_byte_buf part_body,
+    enum aws_s3_checksum_algorithm algorithm,
+    struct aws_byte_buf part_checksum) {
+    AWS_PRECONDITION(allocator);
+
+    if (algorithm == AWS_SCA_NONE) {
+        return AWS_OP_SUCCESS;
+    }
+
+    struct aws_byte_buf checksum;
+    if (aws_byte_buf_init(&checksum, allocator, aws_get_digest_size_from_algorithm(algorithm))) {
+        return AWS_OP_ERR;
+    }
+
+    struct aws_byte_buf encoded_checksum = {0};
+
+    int return_status = AWS_OP_SUCCESS;
+    struct aws_byte_cursor body_cur = aws_byte_cursor_from_buf(&part_body);
+
+    size_t encoded_len = 0;
+    if (aws_base64_compute_encoded_len(aws_get_digest_size_from_algorithm(algorithm), &encoded_len)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "Failed to resume upload. Unable to determine length of encoded checksum.");
+        return_status = aws_raise_error(AWS_ERROR_S3_RESUME_FAILED);
+        goto on_done;
+    }
+
+    if (aws_checksum_compute(allocator, algorithm, &body_cur, &checksum, 0)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "Failed to resume upload. Unable to compute checksum for the skipped part.");
+        return_status = aws_raise_error(AWS_ERROR_S3_RESUME_FAILED);
+        goto on_done;
+    }
+
+    if (aws_byte_buf_init(&encoded_checksum, allocator, encoded_len)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "Failed to resume upload. Unable to allocate buffer for encoded checksum.");
+        return_status = aws_raise_error(AWS_ERROR_S3_RESUME_FAILED);
+        goto on_done;
+    }
+
+    struct aws_byte_cursor checksum_cur = aws_byte_cursor_from_buf(&checksum);
+    if (aws_base64_encode(&checksum_cur, &encoded_checksum)) {
+        AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Failed to resume upload. Unable to encode checksum.");
+        return_status = aws_raise_error(AWS_ERROR_S3_RESUME_FAILED);
+        goto on_done;
+    }
+
+    if (!aws_byte_buf_eq(&encoded_checksum, &part_checksum)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "Failed to resume upload. Checksum for previously uploaded part does not match");
+        return_status = aws_raise_error(AWS_ERROR_S3_RESUMED_PART_CHECKSUM_MISMATCH);
+        goto on_done;
+    }
+
+on_done:
+    aws_byte_buf_clean_up(&checksum);
+    aws_byte_buf_clean_up(&encoded_checksum);
+    return return_status;
+}
+
 /**
  * Skips parts from input stream that were previously uploaded.
  * Assumes input stream has num_parts_read_from_stream specifying which part stream is on
@@ -610,7 +673,9 @@ static int s_skip_parts_from_stream(
     }
 
     struct aws_byte_buf temp_body_buf;
-    aws_byte_buf_init(&temp_body_buf, meta_request->allocator, 0);
+    if (aws_byte_buf_init(&temp_body_buf, meta_request->allocator, 0)) {
+        return AWS_OP_ERR;
+    }
 
     AWS_LOGF_DEBUG(
         AWS_LS_S3_META_REQUEST,
@@ -619,6 +684,7 @@ static int s_skip_parts_from_stream(
         num_parts_read_from_stream,
         skip_until_part_number);
 
+    int return_status = AWS_OP_SUCCESS;
     for (uint32_t part_index = num_parts_read_from_stream; part_index < skip_until_part_number; ++part_index) {
 
         size_t request_body_size = s_compute_request_body_size(meta_request, part_index + 1);
@@ -626,7 +692,9 @@ static int s_skip_parts_from_stream(
         if (temp_body_buf.capacity != request_body_size) {
             // reinit with correct size
             aws_byte_buf_clean_up(&temp_body_buf);
-            aws_byte_buf_init(&temp_body_buf, meta_request->allocator, request_body_size);
+            if (aws_byte_buf_init(&temp_body_buf, meta_request->allocator, request_body_size)) {
+                return AWS_OP_ERR;
+            }
         } else {
             // reuse buffer
             aws_byte_buf_reset(&temp_body_buf, false);
@@ -634,46 +702,25 @@ static int s_skip_parts_from_stream(
 
         if (aws_s3_meta_request_read_body(meta_request, &temp_body_buf)) {
             AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Failed to resume upload. Input steam cannot be read.");
-            aws_byte_buf_clean_up(&temp_body_buf);
-            return AWS_OP_ERR;
+            return_status = AWS_OP_ERR;
+            goto on_done;
         }
 
         // compare skipped checksum to previously uploaded checksum
-        if (meta_request->checksum_config.checksum_algorithm != AWS_SCA_NONE &&
-            auto_ranged_put->checksums_list[part_index].len > 0) {
-            struct aws_byte_buf checksum;
-            aws_byte_buf_init(
-                &checksum,
+        if (auto_ranged_put->encoded_checksum_list[part_index].len > 0 &&
+            s_verify_part_matches_checksum(
                 meta_request->allocator,
-                aws_get_digest_size_from_algorithm(meta_request->checksum_config.checksum_algorithm));
-            struct aws_byte_cursor body_cur = aws_byte_cursor_from_buf(&temp_body_buf);
-
-            if (aws_checksum_compute(
-                    meta_request->allocator,
-                    meta_request->checksum_config.checksum_algorithm,
-                    &body_cur,
-                    &checksum,
-                    0)) {
-                AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "Failed to resume upload. Unable to compute checksum.");
-                aws_byte_buf_clean_up(&temp_body_buf);
-                aws_byte_buf_clean_up(&checksum);
-                return aws_raise_error(AWS_ERROR_S3_RESUME_FAILED);
-            }
-
-            if (!aws_byte_buf_eq(&checksum, &auto_ranged_put->checksums_list[part_index])) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_S3_META_REQUEST,
-                    "Failed to resume upload. Checksum for previously uploaded part does not match "
-                    "new part.");
-                aws_byte_buf_clean_up(&temp_body_buf);
-                aws_byte_buf_clean_up(&checksum);
-                return aws_raise_error(AWS_ERROR_S3_RESUMED_PART_CHECKSUM_MISMATCH);
-            }
+                temp_body_buf,
+                meta_request->checksum_config.checksum_algorithm,
+                auto_ranged_put->encoded_checksum_list[part_index])) {
+            return_status = AWS_OP_ERR;
+            goto on_done;
         }
     }
 
+on_done:
     aws_byte_buf_clean_up(&temp_body_buf);
-    return AWS_OP_SUCCESS;
+    return return_status;
 }
 
 /* Given a request, prepare it for sending based on its description. */
@@ -696,6 +743,12 @@ static int s_s3_auto_ranged_put_prepare_request(
                 aws_s3_meta_request_lock_synced_data(meta_request);
 
                 if (auto_ranged_put->synced_data.list_parts_continuation_token) {
+                    AWS_LOGF_DEBUG(
+                        AWS_LS_S3_META_REQUEST,
+                        "id=%p ListParts for Multi-part Upload, with ID:%s, continues with token:%s.",
+                        (void *)meta_request,
+                        aws_string_c_str(auto_ranged_put->upload_id),
+                        aws_string_c_str(auto_ranged_put->synced_data.list_parts_continuation_token));
                     struct aws_byte_cursor continuation_cur =
                         aws_byte_cursor_from_string(auto_ranged_put->synced_data.list_parts_continuation_token);
                     message_creation_result = aws_s3_construct_next_paginated_request_http_message(
@@ -771,14 +824,14 @@ static int s_s3_auto_ranged_put_prepare_request(
                 auto_ranged_put->upload_id,
                 meta_request->should_compute_content_md5,
                 &meta_request->checksum_config,
-                &auto_ranged_put->checksums_list[request->part_number - 1]);
+                &auto_ranged_put->encoded_checksum_list[request->part_number - 1]);
             break;
         }
         case AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_COMPLETE_MULTIPART_UPLOAD: {
 
             if (request->num_times_prepared == 0) {
 
-                /* Corner case of last part being previosly uploaded during resume.
+                /* Corner case of last part being previously uploaded during resume.
                  * Read it from input stream and potentially verify checksum */
                 if (s_skip_parts_from_stream(
                         meta_request,
@@ -810,7 +863,7 @@ static int s_s3_auto_ranged_put_prepare_request(
                     &request->request_body,
                     auto_ranged_put->upload_id,
                     &auto_ranged_put->synced_data.etag_list,
-                    auto_ranged_put->checksums_list,
+                    auto_ranged_put->encoded_checksum_list,
                     meta_request->checksum_config.checksum_algorithm);
 
                 aws_s3_meta_request_unlock_synced_data(meta_request);
@@ -869,6 +922,31 @@ message_create_failed:
     return AWS_OP_ERR;
 }
 
+/* Invoked before retry */
+static void s_s3_auto_ranged_put_send_request_finish(
+    struct aws_s3_connection *connection,
+    struct aws_http_stream *stream,
+    int error_code) {
+
+    struct aws_s3_request *request = connection->request;
+    AWS_PRECONDITION(request);
+
+    /* Request tag is different from different type of meta requests */
+    switch (request->request_tag) {
+
+        case AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_COMPLETE_MULTIPART_UPLOAD: {
+            /* For complete multipart upload, the server may return async error. */
+            aws_s3_meta_request_send_request_finish_handle_async_error(connection, stream, error_code);
+            break;
+        }
+
+        default:
+            aws_s3_meta_request_send_request_finish_default(connection, stream, error_code);
+            break;
+    }
+}
+
+/* Invoked when no-retry will happen */
 static void s_s3_auto_ranged_put_request_finished(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request,
@@ -892,7 +970,9 @@ static void s_s3_auto_ranged_put_request_finished(
                 if (error_code == AWS_ERROR_SUCCESS) {
 
                     struct aws_byte_cursor body_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
+                    /* Clear the token before */
                     aws_string_destroy(auto_ranged_put->synced_data.list_parts_continuation_token);
+                    auto_ranged_put->synced_data.list_parts_continuation_token = NULL;
                     if (aws_s3_paginated_operation_on_response(
                             auto_ranged_put->synced_data.list_parts_operation,
                             &body_cursor,
@@ -923,11 +1003,31 @@ static void s_s3_auto_ranged_put_request_finished(
                     }
                 }
 
-                auto_ranged_put->synced_data.list_parts_completed = !has_more_results;
-                auto_ranged_put->synced_data.create_multipart_upload_error_code = error_code;
+                if (has_more_results) {
+                    /* If list parts has more result, make sure list parts continues */
+                    auto_ranged_put->synced_data.list_parts_state.continues = true;
+                    auto_ranged_put->synced_data.list_parts_state.completed = false;
+                } else {
+                    /* No more result, complete the list parts */
+                    auto_ranged_put->synced_data.list_parts_state.continues = false;
+                    auto_ranged_put->synced_data.list_parts_state.completed = true;
+                }
+                auto_ranged_put->synced_data.list_parts_error_code = error_code;
 
                 if (error_code != AWS_ERROR_SUCCESS) {
-                    aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
+                    if (request->send_data.response_status == AWS_HTTP_STATUS_CODE_404_NOT_FOUND &&
+                        auto_ranged_put->resume_token->num_parts_completed ==
+                            auto_ranged_put->resume_token->total_num_parts) {
+                        AWS_LOGF_DEBUG(
+                            AWS_LS_S3_META_REQUEST,
+                            "id=%p: Resuming PutObject ended early, since there is nothing to resume"
+                            "(request finished prior to being paused?)",
+                            (void *)meta_request);
+
+                        aws_s3_meta_request_set_success_synced(meta_request, AWS_S3_RESPONSE_STATUS_SUCCESS);
+                    } else {
+                        aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
+                    }
                 }
 
                 aws_s3_meta_request_unlock_synced_data(meta_request);
@@ -941,8 +1041,7 @@ static void s_s3_auto_ranged_put_request_finished(
 
             if (error_code == AWS_ERROR_SUCCESS) {
                 needed_response_headers = aws_http_headers_new(meta_request->allocator);
-                const size_t copy_header_count =
-                    sizeof(s_create_multipart_upload_copy_headers) / sizeof(struct aws_byte_cursor);
+                const size_t copy_header_count = AWS_ARRAY_SIZE(s_create_multipart_upload_copy_headers);
 
                 /* Copy any headers now that we'll need for the final, transformed headers later. */
                 for (size_t header_index = 0; header_index < copy_header_count; ++header_index) {
@@ -950,7 +1049,8 @@ static void s_s3_auto_ranged_put_request_finished(
                     struct aws_byte_cursor header_value;
                     AWS_ZERO_STRUCT(header_value);
 
-                    if (!aws_http_headers_get(request->send_data.response_headers, *header_name, &header_value)) {
+                    if (aws_http_headers_get(request->send_data.response_headers, *header_name, &header_value) ==
+                        AWS_OP_SUCCESS) {
                         aws_http_headers_set(needed_response_headers, *header_name, header_value);
                     }
                 }
@@ -959,7 +1059,7 @@ static void s_s3_auto_ranged_put_request_finished(
 
                 /* Find the upload id for this multipart upload. */
                 struct aws_string *upload_id =
-                    get_top_level_xml_tag_value(meta_request->allocator, &s_upload_id, &buffer_byte_cursor);
+                    aws_xml_get_top_level_tag(meta_request->allocator, &s_upload_id, &buffer_byte_cursor);
 
                 if (upload_id == NULL) {
                     AWS_LOGF_ERROR(
@@ -979,7 +1079,7 @@ static void s_s3_auto_ranged_put_request_finished(
             {
                 aws_s3_meta_request_lock_synced_data(meta_request);
 
-                AWS_ASSERT(auto_ranged_put->synced_data.needed_response_headers == NULL)
+                AWS_ASSERT(auto_ranged_put->synced_data.needed_response_headers == NULL);
                 auto_ranged_put->synced_data.needed_response_headers = needed_response_headers;
 
                 auto_ranged_put->synced_data.create_multipart_upload_completed = true;
@@ -1008,14 +1108,15 @@ static void s_s3_auto_ranged_put_request_finished(
                 AWS_ASSERT(request->send_data.response_headers);
 
                 if (aws_http_headers_get(
-                        request->send_data.response_headers, g_etag_header_name, &etag_within_quotes)) {
+                        request->send_data.response_headers, g_etag_header_name, &etag_within_quotes) !=
+                    AWS_OP_SUCCESS) {
                     AWS_LOGF_ERROR(
                         AWS_LS_S3_META_REQUEST,
                         "id=%p Could not find ETag header for request %p",
                         (void *)meta_request,
                         (void *)request);
 
-                    error_code = AWS_ERROR_S3_MISSING_UPLOAD_ID;
+                    error_code = AWS_ERROR_S3_MISSING_ETAG;
                 } else {
                     /* The ETag value arrives in quotes, but we don't want it in quotes when we send it back up
                      * later, so just get rid of the quotes now. */
@@ -1094,7 +1195,7 @@ static void s_s3_auto_ranged_put_request_finished(
 
                 /* Grab the ETag for the entire object, and set it as a header. */
                 struct aws_string *etag_header_value =
-                    get_top_level_xml_tag_value(meta_request->allocator, &g_etag_header_name, &response_body_cursor);
+                    aws_xml_get_top_level_tag(meta_request->allocator, &g_etag_header_name, &response_body_cursor);
 
                 if (etag_header_value != NULL) {
                     struct aws_byte_buf etag_header_value_byte_buf;
@@ -1154,62 +1255,41 @@ static void s_s3_auto_ranged_put_request_finished(
     }
 }
 
-static int s_s3_auto_ranged_put_pause(struct aws_s3_meta_request *meta_request, struct aws_string **out_resume_token) {
+static int s_s3_auto_ranged_put_pause(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_meta_request_resume_token **out_resume_token) {
+
+    *out_resume_token = NULL;
 
     /* lock */
     aws_s3_meta_request_lock_synced_data(meta_request);
     struct aws_s3_auto_ranged_put *auto_ranged_put = meta_request->impl;
 
-    /* only generate token if multipart upload was completed, since there is nothing to resume otherwise*/
-    int token_generated_error = AWS_OP_SUCCESS;
-    *out_resume_token = NULL;
+    AWS_LOGF_DEBUG(
+        AWS_LS_S3_META_REQUEST,
+        "id=%p: Pausing request with %u out of %u parts have completed.",
+        (void *)meta_request,
+        auto_ranged_put->synced_data.num_parts_completed,
+        auto_ranged_put->synced_data.total_num_parts);
+
+    /* upload can be in one of several states:
+     * - not started, i.e. we didn't even call crete mpu yet - return success,
+     *   token is NULL and cancel the upload
+     * - in the middle of upload - return success, create token and cancel
+     *     upload
+     * - complete MPU started - return success, generate token and try to cancel
+     *   complete MPU
+     */
     if (auto_ranged_put->synced_data.create_multipart_upload_completed) {
-        struct aws_json_value *root = aws_json_value_new_object(meta_request->allocator);
 
-        /**
-         * generate pause token for put.
-         * current format is json with the following structure
-         * {
-         *      type: String
-         *      multipart_upload_id: String
-         *      partition_size: number
-         *      total_num_parts: number
-         * }
-         */
+        *out_resume_token = aws_s3_meta_request_resume_token_new(meta_request->allocator);
 
-        aws_json_value_add_to_object(
-            root,
-            aws_byte_cursor_from_c_str("type"),
-            aws_json_value_new_string(
-                meta_request->allocator, aws_byte_cursor_from_c_str("AWS_S3_META_REQUEST_TYPE_PUT_OBJECT")));
-
-        aws_json_value_add_to_object(
-            root,
-            aws_byte_cursor_from_c_str("multipart_upload_id"),
-            aws_json_value_new_string(
-                meta_request->allocator, aws_byte_cursor_from_string(auto_ranged_put->upload_id)));
-
-        aws_json_value_add_to_object(
-            root,
-            aws_byte_cursor_from_c_str("partition_size"),
-            aws_json_value_new_number(meta_request->allocator, (double)meta_request->part_size));
-
-        aws_json_value_add_to_object(
-            root,
-            aws_byte_cursor_from_c_str("total_num_parts"),
-            aws_json_value_new_number(meta_request->allocator, (double)auto_ranged_put->synced_data.total_num_parts));
-
-        struct aws_byte_buf result_string_buf;
-        aws_byte_buf_init(&result_string_buf, meta_request->allocator, 0);
-
-        token_generated_error = aws_byte_buf_append_json_string(root, &result_string_buf);
-
-        if (!token_generated_error) {
-            *out_resume_token = aws_string_new_from_buf(meta_request->allocator, &result_string_buf);
-        }
-
-        aws_byte_buf_clean_up(&result_string_buf);
-        aws_json_value_destroy(root);
+        (*out_resume_token)->type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT;
+        (*out_resume_token)->multipart_upload_id =
+            aws_string_clone_or_reuse(meta_request->allocator, auto_ranged_put->upload_id);
+        (*out_resume_token)->part_size = meta_request->part_size;
+        (*out_resume_token)->total_num_parts = auto_ranged_put->synced_data.total_num_parts;
+        (*out_resume_token)->num_parts_completed = auto_ranged_put->synced_data.num_parts_completed;
     }
 
     /**
@@ -1221,5 +1301,5 @@ static int s_s3_auto_ranged_put_pause(struct aws_s3_meta_request *meta_request,
     /* unlock */
     aws_s3_meta_request_unlock_synced_data(meta_request);
 
-    return token_generated_error;
+    return AWS_OP_SUCCESS;
 }
