@@ -77,6 +77,10 @@ static const double s_default_throughput_target_gbps = 10.0;
 static const uint32_t s_default_max_retries = 5;
 static size_t s_dns_host_address_ttl_seconds = 5 * 60;
 
+/* Default time until a connection is declared dead, while handling a request but seeing no activity.
+ * 30 seconds mirrors the value currently used by the Java SDK. */
+static const uint32_t s_default_throughput_failure_interval_seconds = 30;
+
 /* Called when ref count is 0. */
 static void s_s3_client_start_destroy(void *user_data);
 
@@ -325,9 +329,11 @@ struct aws_s3_client *aws_s3_client_new(
     }
 
     if (client_config->monitoring_options) {
-        client->monitoring_options =
-            aws_mem_calloc(allocator, 1, sizeof(struct aws_http_connection_monitoring_options));
-        *client->monitoring_options = *client_config->monitoring_options;
+        client->monitoring_options = *client_config->monitoring_options;
+    } else {
+        client->monitoring_options.minimum_throughput_bytes_per_second = 1;
+        client->monitoring_options.allowable_throughput_failure_interval_seconds =
+            s_default_throughput_failure_interval_seconds;
     }
 
     if (client_config->tls_mode == AWS_MR_TLS_ENABLED) {
@@ -469,7 +475,6 @@ on_error:
         client->proxy_ev_settings->tls_options = NULL;
     }
     aws_mem_release(client->allocator, client->proxy_ev_settings);
-    aws_mem_release(client->allocator, client->monitoring_options);
     aws_mem_release(client->allocator, client->tcp_keep_alive_options);
 
     aws_event_loop_group_release(client->client_bootstrap->event_loop_group);
@@ -557,7 +562,6 @@ static void s_s3_client_finish_destroy_default(struct aws_s3_client *client) {
         client->proxy_ev_settings->tls_options = NULL;
     }
     aws_mem_release(client->allocator, client->proxy_ev_settings);
-    aws_mem_release(client->allocator, client->monitoring_options);
     aws_mem_release(client->allocator, client->tcp_keep_alive_options);
 
     aws_mutex_clean_up(&client->synced_data.lock);
@@ -859,7 +863,8 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
                 .proxy_ev_settings = client->proxy_ev_settings,
                 .connect_timeout_ms = client->connect_timeout_ms,
                 .tcp_keep_alive_options = client->tcp_keep_alive_options,
-                .monitoring_options = client->monitoring_options};
+                .monitoring_options = &client->monitoring_options,
+            };
 
             endpoint = aws_s3_endpoint_new(client->allocator, &endpoint_options);
 
@@ -969,9 +974,10 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
 
             struct aws_input_stream *input_stream = aws_http_message_get_body_stream(options->message);
 
-            if (input_stream == NULL) {
+            if ((input_stream == NULL) && (options->send_filepath.len == 0)) {
                 AWS_LOGF_ERROR(
-                    AWS_LS_S3_META_REQUEST, "Could not create auto-ranged-put meta request; body stream is NULL.");
+                    AWS_LS_S3_META_REQUEST,
+                    "Could not create auto-ranged-put meta request; filepath or body stream must be set.");
                 aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
                 return NULL;
             }
