@@ -7,6 +7,7 @@
 #include "aws/s3/private/s3_checksums.h"
 #include "aws/s3/private/s3_client_impl.h"
 #include "aws/s3/private/s3_meta_request_impl.h"
+#include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
 #include <aws/auth/signable.h>
 #include <aws/auth/signing.h>
@@ -215,10 +216,6 @@ int aws_s3_meta_request_init_base(
         goto error;
     }
 
-    /* Nothing can fail after here. Leave the impl not affected by failure of initializing base. */
-    meta_request->impl = impl;
-    meta_request->vtable = vtable;
-
     *((size_t *)&meta_request->part_size) = part_size;
     *((bool *)&meta_request->should_compute_content_md5) = should_compute_content_md5;
     checksum_config_init(&meta_request->checksum_config, options->checksum_config);
@@ -226,9 +223,18 @@ int aws_s3_meta_request_init_base(
         meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
     }
 
-    /* Keep a reference to the original message structure passed in. */
-    meta_request->initial_request_message = options->message;
-    aws_http_message_acquire(options->message);
+    /* Set initial_meta_request */
+    if (options->send_filepath.len > 0) {
+        /* Create copy of original message, but with body-stream that reads directly from file */
+        meta_request->initial_request_message = aws_s3_message_util_copy_http_message_filepath_body_all_headers(
+            allocator, options->message, options->send_filepath);
+        if (meta_request->initial_request_message == NULL) {
+            goto error;
+        }
+    } else {
+        /* Keep a reference to the original message structure passed in. */
+        meta_request->initial_request_message = aws_http_message_acquire(options->message);
+    }
 
     /* Client is currently optional to allow spinning up a meta_request without a client in a test. */
     if (client != NULL) {
@@ -260,6 +266,10 @@ int aws_s3_meta_request_init_base(
         meta_request->finish_callback = options->finish_callback;
     }
 
+    /* Nothing can fail after here. Leave the impl not affected by failure of initializing base. */
+    meta_request->impl = impl;
+    meta_request->vtable = vtable;
+
     return AWS_OP_SUCCESS;
 error:
     s_s3_meta_request_destroy((void *)meta_request);
