@@ -22,6 +22,7 @@ const struct aws_byte_cursor g_s3_client_version = AWS_BYTE_CUR_INIT_FROM_STRING
 const struct aws_byte_cursor g_s3_service_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("s3");
 const struct aws_byte_cursor g_host_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Host");
 const struct aws_byte_cursor g_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Range");
+const struct aws_byte_cursor g_if_match_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("If-Match");
 const struct aws_byte_cursor g_etag_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ETag");
 const struct aws_byte_cursor g_content_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Range");
 const struct aws_byte_cursor g_content_type_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Type");
@@ -60,6 +61,14 @@ const struct aws_byte_cursor g_user_agent_header_name = AWS_BYTE_CUR_INIT_FROM_S
 const struct aws_byte_cursor g_user_agent_header_product_name =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("CRTS3NativeClient");
 
+const struct aws_byte_cursor g_error_body_xml_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Error");
+const struct aws_byte_cursor g_code_body_xml_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Code");
+
+const struct aws_byte_cursor g_s3_internal_error_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("InternalError");
+const struct aws_byte_cursor g_s3_slow_down_error_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("SlowDown");
+/* The special error code as Asynchronous Error Codes */
+const struct aws_byte_cursor g_s3_internal_errors_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("InternalErrors");
+
 const uint32_t g_s3_max_num_upload_parts = 10000;
 const size_t g_s3_min_upload_part_size = MB_TO_BYTES(5);
 
@@ -77,9 +86,11 @@ void copy_http_headers(const struct aws_http_headers *src, struct aws_http_heade
     }
 }
 
-struct top_level_xml_tag_value_user_data {
+struct top_level_xml_tag_value_with_root_value_user_data {
     struct aws_allocator *allocator;
     const struct aws_byte_cursor *tag_name;
+    const struct aws_byte_cursor *expected_root_name;
+    bool *root_name_mismatch;
     struct aws_string *result;
 };
 
@@ -95,7 +106,7 @@ static bool s_top_level_xml_tag_value_child_xml_node(
         return false;
     }
 
-    struct top_level_xml_tag_value_user_data *xml_user_data = user_data;
+    struct top_level_xml_tag_value_with_root_value_user_data *xml_user_data = user_data;
 
     /* If the name of the node is what we are looking for, store the body of the node in our result, and stop
      * traversing. */
@@ -117,15 +128,30 @@ static bool s_top_level_xml_tag_value_root_xml_node(
     struct aws_xml_parser *parser,
     struct aws_xml_node *node,
     void *user_data) {
+    struct top_level_xml_tag_value_with_root_value_user_data *xml_user_data = user_data;
+    if (xml_user_data->expected_root_name) {
+        /* If we can't get the name of the node, stop traversing. */
+        struct aws_byte_cursor node_name;
+        if (aws_xml_node_get_name(node, &node_name)) {
+            return false;
+        }
+        if (!aws_byte_cursor_eq(&node_name, xml_user_data->expected_root_name)) {
+            /* Not match the expected root name, stop parsing. */
+            *xml_user_data->root_name_mismatch = true;
+            return false;
+        }
+    }
 
     /* Traverse the root node, and then return false to stop. */
     aws_xml_node_traverse(parser, node, s_top_level_xml_tag_value_child_xml_node, user_data);
     return false;
 }
 
-struct aws_string *get_top_level_xml_tag_value(
+struct aws_string *aws_xml_get_top_level_tag_with_root_name(
     struct aws_allocator *allocator,
     const struct aws_byte_cursor *tag_name,
+    const struct aws_byte_cursor *expected_root_name,
+    bool *out_root_name_mismatch,
     struct aws_byte_cursor *xml_body) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(tag_name);
@@ -133,10 +159,13 @@ struct aws_string *get_top_level_xml_tag_value(
 
     struct aws_xml_parser_options parser_options = {.doc = *xml_body};
     struct aws_xml_parser *parser = aws_xml_parser_new(allocator, &parser_options);
+    bool root_name_mismatch = false;
 
-    struct top_level_xml_tag_value_user_data xml_user_data = {
+    struct top_level_xml_tag_value_with_root_value_user_data xml_user_data = {
         allocator,
         tag_name,
+        expected_root_name,
+        &root_name_mismatch,
         NULL,
     };
 
@@ -145,6 +174,9 @@ struct aws_string *get_top_level_xml_tag_value(
         xml_user_data.result = NULL;
         goto clean_up;
     }
+    if (out_root_name_mismatch) {
+        *out_root_name_mismatch = root_name_mismatch;
+    }
 
 clean_up:
 
@@ -153,6 +185,13 @@ clean_up:
     return xml_user_data.result;
 }
 
+struct aws_string *aws_xml_get_top_level_tag(
+    struct aws_allocator *allocator,
+    const struct aws_byte_cursor *tag_name,
+    struct aws_byte_cursor *xml_body) {
+    return aws_xml_get_top_level_tag_with_root_name(allocator, tag_name, NULL, NULL, xml_body);
+}
+
 struct aws_cached_signing_config_aws *aws_cached_signing_config_new(
     struct aws_allocator *allocator,
     const struct aws_signing_config_aws *signing_config) {
@@ -439,10 +478,6 @@ int aws_s3_parse_content_length_response_header(
 }
 
 uint32_t aws_s3_get_num_parts(size_t part_size, uint64_t object_range_start, uint64_t object_range_end) {
-    if ((object_range_start - object_range_end) == 0ULL) {
-        return 0;
-    }
-
     uint32_t num_parts = 1;
 
     uint64_t first_part_size = part_size;
@@ -457,7 +492,7 @@ uint32_t aws_s3_get_num_parts(size_t part_size, uint64_t object_range_start, uin
 
     /* If the range has room for a second part, calculate the additional amount of parts. */
     if (second_part_start <= object_range_end) {
-        uint64_t aligned_range_remainder = object_range_end - second_part_start;
+        uint64_t aligned_range_remainder = object_range_end + 1 - second_part_start;
         num_parts += (uint32_t)(aligned_range_remainder / (uint64_t)part_size);
 
         if ((aligned_range_remainder % part_size) > 0) {
@@ -510,3 +545,14 @@ void aws_s3_get_part_range(
         *out_part_range_end = object_range_end;
     }
 }
+
+int aws_s3_crt_error_code_from_server_error_code_string(const struct aws_string *error_code_string) {
+    if (aws_string_eq_byte_cursor(error_code_string, &g_s3_slow_down_error_code)) {
+        return AWS_ERROR_S3_SLOW_DOWN;
+    }
+    if (aws_string_eq_byte_cursor(error_code_string, &g_s3_internal_error_code) ||
+        aws_string_eq_byte_cursor(error_code_string, &g_s3_internal_errors_code)) {
+        return AWS_ERROR_S3_INTERNAL_ERROR;
+    }
+    return AWS_ERROR_UNKNOWN;
+}
