@@ -160,7 +160,14 @@ int i2c_ASN1_INTEGER(const ASN1_INTEGER *in, unsigned char **outp) {
   if (pad) {
     (*outp)[0] = 0;
   }
-  OPENSSL_memcpy(*outp + pad, in->data + start, in->length - start);
+  // If in->data is null the Undefined Behavior Sanitior flags this as applying
+  // an offset to a null pointer. Gracefully handle the case even though
+  // OPENSSL_memcpy handles the case when in->data is null and in->length is zero.
+  // Don't return early because an empty integer is still encoded as a single
+  // below byte.
+  if (in->data != NULL) {
+    OPENSSL_memcpy(*outp + pad, in->data + start, in->length - start);
+  }
   if (is_negative) {
     negate_twos_complement(*outp, len);
     assert((*outp)[0] >= 0x80);
@@ -246,7 +253,7 @@ err:
   return NULL;
 }
 
-int ASN1_INTEGER_set(ASN1_INTEGER *a, long v) {
+int ASN1_INTEGER_set_int64(ASN1_INTEGER *a, int64_t v) {
   if (v >= 0) {
     return ASN1_INTEGER_set_uint64(a, (uint64_t)v);
   }
@@ -259,7 +266,7 @@ int ASN1_INTEGER_set(ASN1_INTEGER *a, long v) {
   return 1;
 }
 
-int ASN1_ENUMERATED_set(ASN1_ENUMERATED *a, long v) {
+int ASN1_ENUMERATED_set_int64(ASN1_ENUMERATED *a, int64_t v) {
   if (v >= 0) {
     return ASN1_ENUMERATED_set_uint64(a, (uint64_t)v);
   }
@@ -272,6 +279,16 @@ int ASN1_ENUMERATED_set(ASN1_ENUMERATED *a, long v) {
   return 1;
 }
 
+int ASN1_INTEGER_set(ASN1_INTEGER *a, long v) {
+  OPENSSL_STATIC_ASSERT(sizeof(long) <= sizeof(int64_t), long_fits_in_int64_t);
+  return ASN1_INTEGER_set_int64(a, v);
+}
+
+int ASN1_ENUMERATED_set(ASN1_ENUMERATED *a, long v) {
+  OPENSSL_STATIC_ASSERT(sizeof(long) <= sizeof(int64_t), long_fits_in_int64_t);
+  return ASN1_ENUMERATED_set_int64(a, v);
+}
+
 static int asn1_string_set_uint64(ASN1_STRING *out, uint64_t v, int type) {
   uint8_t buf[sizeof(uint64_t)];
   CRYPTO_store_u64_be(buf, v);
@@ -333,16 +350,11 @@ int ASN1_ENUMERATED_get_uint64(uint64_t *out, const ASN1_ENUMERATED *a) {
   return asn1_string_get_uint64(out, a, V_ASN1_ENUMERATED);
 }
 
-static long asn1_string_get_long(const ASN1_STRING *a, int type) {
-  if (a == NULL) {
-    return 0;
-  }
-
+static int asn1_string_get_int64(int64_t *out, const ASN1_STRING *a, int type) {
   uint64_t v;
   if (!asn1_string_get_abs_uint64(&v, a, type)) {
-    goto err;
+    return 0;
   }
-
   int64_t i64;
   int fits_in_i64;
   // Check |v != 0| to handle manually-constructed negative zeros.
@@ -353,16 +365,36 @@ static long asn1_string_get_long(const ASN1_STRING *a, int type) {
     i64 = (int64_t)v;
     fits_in_i64 = i64 >= 0;
   }
-  OPENSSL_STATIC_ASSERT(sizeof(long) <= sizeof(int64_t), long_is_too_big)
+  if (!fits_in_i64) {
+    OPENSSL_PUT_ERROR(ASN1, ASN1_R_INVALID_INTEGER);
+    return 0;
+  }
+  *out = i64;
+  return 1;
+}
 
-  if (fits_in_i64 && LONG_MIN <= i64 && i64 <= LONG_MAX) {
-    return (long)i64;
+int ASN1_INTEGER_get_int64(int64_t *out, const ASN1_INTEGER *a) {
+  return asn1_string_get_int64(out, a, V_ASN1_INTEGER);
+}
+
+int ASN1_ENUMERATED_get_int64(int64_t *out, const ASN1_ENUMERATED *a) {
+  return asn1_string_get_int64(out, a, V_ASN1_ENUMERATED);
+}
+
+static long asn1_string_get_long(const ASN1_STRING *a, int type) {
+  if (a == NULL) {
+    return 0;
   }
 
-err:
-  // This function's return value does not distinguish overflow from -1.
-  ERR_clear_error();
-  return -1;
+  int64_t v;
+  if (!asn1_string_get_int64(&v, a, type) ||  //
+      v < LONG_MIN || v > LONG_MAX) {
+    // This function's return value does not distinguish overflow from -1.
+    ERR_clear_error();
+    return -1;
+  }
+
+  return (long)v;
 }
 
 long ASN1_INTEGER_get(const ASN1_INTEGER *a) {
