@@ -15,6 +15,7 @@
 #include <gtest/gtest.h>
 #include <stdlib.h>
 
+#include <openssl/ctrdrbg.h>
 #include <openssl/rand.h>
 
 #include "internal.h"
@@ -533,10 +534,6 @@ int main(int argc, char **argv) {
     CRYPTO_fork_detect_ignore_madv_wipeonfork_for_testing();
   }
 
-  if (getenv("BORINGSSL_IGNORE_PTHREAD_ATFORK")) {
-    CRYPTO_fork_detect_ignore_pthread_atfork_for_testing();
-  }
-
   return RUN_ALL_TESTS();
 }
 
