@@ -208,7 +208,7 @@ extern "C" {
 // against multiple revisions of BoringSSL at the same time. It is not
 // recommended to do so for longer than is necessary.
 
-#define AWSLC_API_VERSION 20
+#define AWSLC_API_VERSION 21
 
 // This string tracks the most current production release version on Github
 // https://github.com/aws/aws-lc/releases.
@@ -216,7 +216,7 @@ extern "C" {
 // ServiceIndicatorTest.AWSLCVersionString
 // Note: there are two versions of this test. Only one test is compiled
 // depending on FIPS mode.
-#define AWSLC_VERSION_NUMBER_STRING "1.5.0"
+#define AWSLC_VERSION_NUMBER_STRING "1.8.0"
 
 #if defined(BORINGSSL_SHARED_LIBRARY)
 
@@ -480,6 +480,17 @@ typedef struct x509_trust_st X509_TRUST;
 
 typedef void *OPENSSL_BLOCK;
 
+#ifndef __has_attribute
+#define __has_attribute(x) 0
+#endif
+#if __STDC_VERSION__ > 202300L
+#define AWS_LC_DEPRECATED [[deprecated]]
+#elif __has_attribute(deprecated)
+#define AWS_LC_DEPRECATED __attribute__((deprecated))
+#else
+#define AWS_LC_DEPRECATED /* deprecated */
+#endif
+
 #if defined(__cplusplus)
 }  // extern C
 #elif !defined(BORINGSSL_NO_CXX)
