@@ -81,6 +81,7 @@ struct s2n_cipher {
         struct s2n_composite_cipher comp;
     } io;
     uint8_t key_material_size;
+    bool ktls_supported;
     uint8_t (*is_available)(void);
     int (*init)(struct s2n_session_key *key);
     int (*set_decryption_key)(struct s2n_session_key *key, struct s2n_blob *in);
@@ -88,8 +89,8 @@ struct s2n_cipher {
     int (*destroy_key)(struct s2n_session_key *key);
 };
 
-extern int s2n_session_key_alloc(struct s2n_session_key *key);
-extern int s2n_session_key_free(struct s2n_session_key *key);
+int s2n_session_key_alloc(struct s2n_session_key *key);
+int s2n_session_key_free(struct s2n_session_key *key);
 
 extern const struct s2n_cipher s2n_null_cipher;
 extern const struct s2n_cipher s2n_rc4;
